% Nan Xiao

% 6/29/05

% assemble M K L matrices in the 'Hines' method

function [CaRHS] = neur_assembleCaRHS(nc,md,p_const,p_conds,y,ILp) 

root = md.root;
rootdex = md.rootdex;
notroot = md.notroot;
notrootdex = md.notrootdex;

CaCaBcen = cell(md.count.N,1);
CaCaEcen = cell(md.count.N,1);
ICacen = cell(md.count.N,1);

for jj = 1:md.count.N
    
    h = md.grid.hstep(jj);
    
    ai  = y.Ca{jj}';
    aim = [0 ai(1:end-1)];
    aip = [ai(2:end) 0];
    
    bi  = y.CaB{jj}';
    bim = [0 bi(1:end-1)];
    bip = [bi(2:end) 0];
    
    ci  = y.CaE{jj}';
    cim = [0 ci(1:end-1)];
    cip = [ci(2:end) 0];
    
    di = ILp{jj}(:,end)'./md.interp.radii{jj};
    dim = [0 di(1:end-1)];
    dip = [di(2:end) 0];
      
    CaCaB1 = h/12*(aim.*(bim+bi)+ai.*(3*bi+bim));
    CaCaB2 = h/12*(aip.*(bip+bi)+ai.*(3*bi+bip));
    
    CaCaE1 = h/12*(aim.*(cim+ci)+ai.*(3*ci+cim));
    CaCaE2 = h/12*(aip.*(cip+ci)+ai.*(3*ci+cip));
    
    ICa1 = h/6*(2*di+dim);
    ICa2 = h/6*(2*di+dip);
             
    ICacen{jj} = ([0 ICa1(2:end)] + [ICa2(1:end-1) 0]);
    
    CaCaBcen{jj} = ([0 CaCaB1(2:end)] + [CaCaB2(1:end-1) 0]);
    CaCaEcen{jj} = ([0 CaCaE1(2:end)] + [CaCaE2(1:end-1) 0]);
        
end

% first root is designated as 'master root'

CaCaB = zeros(md.count.L,1);
CaCaE = zeros(md.count.L,1);
ICa = zeros(md.count.L,1);

jj = rootdex(end);

segh = md.segh(jj);
segt = md.segt(jj);

CaCaB(segh:segt) = CaCaBcen{jj};
CaCaE(segh:segt) = CaCaEcen{jj};
ICa(segh:segt) = ICacen{jj};

% rest of the roots

for ii = 1:length(rootdex)-1

    jj = rootdex(ii);
    
    CaCaB(end) = CaCaB(end) + CaCaBcen{jj}(end);
    CaCaE(end) = CaCaE(end) + CaCaEcen{jj}(end);
    ICa(end) = ICa(end) + ICacen{jj}(end);
       
end

% rest of the branches

for ii = 1:length(notrootdex)
   
    jj = notrootdex(ii);
    
    segh = md.segh(jj);
    segt = md.segt(jj);
        
    CaCaB(segh:segt) = CaCaBcen{jj}(1:end-1);
    CaCaE(segh:segt) = CaCaEcen{jj}(1:end-1);
    ICa(segh:segt) = ICacen{jj}(1:end-1);
    
end

% connections

for ii = 1:length(md.par)
   
    jj = md.par(ii);
    
    segh = md.segh(jj);
    d1 = nc.treedata{jj}(1);
    d2 = nc.treedata{jj}(2);
         
    CaCaB(segh) = CaCaB(segh) + CaCaBcen{d1}(end) + CaCaBcen{d2}(end);
    CaCaE(segh) = CaCaE(segh) + CaCaEcen{d1}(end) + CaCaEcen{d2}(end);    
    ICa(segh) = ICa(segh) + ICacen{d1}(end) + ICacen{d2}(end);    
end

kBp = p_const.Ca.kBp;
kEp = p_const.Ca.kEp;
kBm = p_const.Ca.kBm;
kEm = p_const.Ca.kEm;


% make hybrid-Euler RHS vector

CaRHS = [kBp*CaCaB+kEp*CaCaE-ICa/p_const.F; -kBp*CaCaB; -kEp*CaCaE];
