% Nan Xiao

% 6/29/05

% assemble M K L matrices in the 'Hines' method

function [M,K,L,b,md] = neur_assembleMKh(nc,md,p_const,p_conds) 

% set up up the data

[md.segh,md.segt,root] = neur_headtailh(nc,md);

rootdex = nc.roots;
notroot = abs(root-1);
notrootdex = find(notroot);

md.root = root;
md.rootdex = rootdex;
md.notroot = notroot;
md.notrootdex = notrootdex;

M = spalloc(md.count.L,md.count.L,md.count.L*3-4);
K = spalloc(md.count.L,md.count.L,md.count.L*3-4);
L = spalloc(md.count.L,md.count.L,md.count.L*3-4);
b = zeros(md.count.L,1);

Kcen = cell(md.count.N,1);
Mcen = cell(md.count.N,1);
Lcen = cell(md.count.N,1);
Koff = cell(md.count.N,1);
Moff = cell(md.count.N,1);
Loff = cell(md.count.N,1);
bcen = cell(md.count.N,1);

for jj = 1:md.count.N
    
    %md.interp.radii{jj} = md.interp.radii{jj}*1e-4;
    
    h = md.grid.hstep(jj);
    
    ai  = md.interp.radii{jj};
    aim = [0 ai(1:end-1)];
    aip = [ai(2:end) 0];
    
    Gmi = [p_conds.pas.G{jj} 1];
    Gmim = [1 Gmi(1:end-1)];
    
    Kcen{jj} = 1/(3*h)*(2*ai.^2+aim.^2+aip.^2+ai.*(aim+aip));
    Kcen{jj}(1) = 1/(3*h)*(ai(1).^2+aip(1).^2+ai(1).*aip(1));
    Kcen{jj}(end) = 1/(3*h)*(ai(end).^2+aim(end).^2+ai(end).*aim(end));
    
    M1 = 1/12*(aim+3*ai).*sqrt(h^2+(aim-ai).^2);
    M2 = 1/12*(aip+3*ai).*sqrt(h^2+(aip-ai).^2);
    
    Mcen{jj} = [0 M1(2:end)] + [M2(1:end-1) 0];
    
    Lcen{jj} = Gmim.*[0 M1(2:end)] + Gmi.*[M2(1:end-1) 0];
    
    b1 = 1/12*(2*aim+4*ai).*sqrt(h^2+(aim-ai).^2);
    b2 = 1/12*(2*aip+4*ai).*sqrt(h^2+(aip-ai).^2);
    
    bcen{jj} = p_const.pas.v*(Gmim.*[0 b1(2:end)] + ...
                              Gmi.*[b2(1:end-1) 0]);
    
    aip = ai(2:end);
    ai = ai(1:end-1);
    Gmi = Gmi(1:end-1);
    
    Koff{jj} = -1/(3*h)*(ai.^2+ai.*aip+aip.^2);
    Moff{jj} = 1/12*(ai+aip).*sqrt(h^2+(ai-aip).^2);
    Loff{jj} = Gmi.*Moff{jj};
    
end

% first root is designated as 'master root'

jj = rootdex(end);

segh = md.segh(jj);
segt = md.segt(jj);

st = md.count.L*(segh-1)+segh;
fin = md.count.L*(segt-1)+segt;
step = md.count.L+1;

K(st:step:fin) = Kcen{jj};
K(st+1:step:fin-md.count.L) = Koff{jj};
K(st+md.count.L:step:fin-1) = Koff{jj};

M(st:step:fin) = Mcen{jj};
M(st+1:step:fin-md.count.L) = Moff{jj};
M(st+md.count.L:step:fin-1) = Moff{jj};

L(st:step:fin) = Lcen{jj};
L(st+1:step:fin-md.count.L) = Loff{jj};
L(st+md.count.L:step:fin-1) = Loff{jj};

b(segh:segt) = bcen{jj};

% rest of the roots

for ii = 1:length(rootdex)-1

    jj = rootdex(ii);
    
    segh = md.segh(jj);
    segt = md.segt(jj);
    
    st = md.count.L*(segh-1)+segh;
    fin = md.count.L*(segt-1)+segt;
    step = md.count.L+1;
    
    K(st:step:fin) = Kcen{jj}(1:end-1);
    K(st+1:step:fin-md.count.L) = Koff{jj}(1:end-1);
    K(st+md.count.L:step:fin-1) = Koff{jj}(1:end-1);
        
    M(st:step:fin) = Mcen{jj}(1:end-1);
    M(st+1:step:fin-md.count.L) = Moff{jj}(1:end-1);
    M(st+md.count.L:step:fin-1) = Moff{jj}(1:end-1);
    
    L(st:step:fin) = Lcen{jj}(1:end-1);
    L(st+1:step:fin-md.count.L) = Loff{jj}(1:end-1);
    L(st+md.count.L:step:fin-1) = Loff{jj}(1:end-1);
    
    b(segh:segt) = bcen{jj}(1:end-1);
    
    K(end,end) = K(end,end) + Kcen{jj}(end);
    K(end,segt) = Koff{jj}(end);
    K(segt,end) = Koff{jj}(end);
    
    M(end,end) = M(end,end) + Mcen{jj}(end);
    M(end,segt) = Moff{jj}(end);
    M(segt,end) = Moff{jj}(end);
    
    L(end,end) = L(end,end) + Lcen{jj}(end);
    L(end,segt) = Loff{jj}(end);
    L(segt,end) = Loff{jj}(end);
    
    b(end) = b(end) + bcen{jj}(end);
end

% rest of the branches

for ii = 1:length(notrootdex)
   
    jj = notrootdex(ii);
    
    segh = md.segh(jj);
    segt = md.segt(jj);
    
    st = md.count.L*(segh-1)+segh;
    fin = md.count.L*(segt-1)+segt;
    step = md.count.L+1;
    
    K(st:step:fin) = Kcen{jj}(1:end-1);
    K(st+1:step:fin-md.count.L) = Koff{jj}(1:end-1);
    K(st+md.count.L:step:fin-1) = Koff{jj}(1:end-1);
    
    M(st:step:fin) = Mcen{jj}(1:end-1);
    M(st+1:step:fin-md.count.L) = Moff{jj}(1:end-1);
    M(st+md.count.L:step:fin-1) = Moff{jj}(1:end-1);
    
    L(st:step:fin) = Lcen{jj}(1:end-1);
    L(st+1:step:fin-md.count.L) = Loff{jj}(1:end-1);
    L(st+md.count.L:step:fin-1) = Loff{jj}(1:end-1);
    
    b(segh:segt) = bcen{jj}(1:end-1);
    
end

% connections

for ii = 1:length(md.par)
   
    jj = md.par(ii);
    
    segh = md.segh(jj);
    d1 = nc.treedata{jj}(1);
    d2 = nc.treedata{jj}(2);
    
    segt1 = md.segt(d1);
    segt2 = md.segt(d2);
    
    K(segh,segh) = K(segh,segh) + Kcen{d1}(end) + Kcen{d2}(end); 
    K(segh,segt1) = Koff{d1}(end);
    K(segt1,segh) = Koff{d1}(end);
    K(segh,segt2) = Koff{d2}(end);
    K(segt2,segh) = Koff{d2}(end);
    
    M(segh,segh) = M(segh,segh) + Mcen{d1}(end) + Mcen{d2}(end); 
    M(segh,segt1) = Moff{d1}(end);
    M(segt1,segh) = Moff{d1}(end);
    M(segh,segt2) = Moff{d2}(end);
    M(segt2,segh) = Moff{d2}(end);
    
    L(segh,segh) = L(segh,segh) + Lcen{d1}(end) + Lcen{d2}(end); 
    L(segh,segt1) = Loff{d1}(end);
    L(segt1,segh) = Loff{d1}(end);
    L(segh,segt2) = Loff{d2}(end);
    L(segt2,segh) = Loff{d2}(end);
    
    b(segh) = b(segh) + bcen{d1}(end) + bcen{d2}(end);
    
end

M(end,end) = M(end,end) + p_const.soma.As;
L(end,end) = L(end,end) + p_const.soma.As*p_conds.pas.GS;
b(end) = b(end) + p_const.soma.As*p_conds.pas.GS*p_const.pas.v;

% K = K*1e-4*1/(2*p_const.pas.Ra);
% M = M*1e-8*p_const.pas.Cm;
% L = L*1e-8;
% b = b*1e-8;

K = K*1/(2*p_const.pas.Ra);
M = M*p_const.pas.Cm;
%L = L;
%b = b;

function [segh, segt, root] = neur_headtailh(nc,md)

segt = zeros(md.count.N,1);
segh = zeros(md.count.N,1);
root = zeros(md.count.N,1);

segh(1) = 1;

% tempsize(1) = md.interp.size(1);
% segt(1) = tempsize(1);
% segh(2) = segt(1)+1;

% root(1) = 1;

for ii = 1:md.count.N-1

    % make tempsize array (adjust for triple-click)
    
    tempsize(ii) = md.interp.size(ii)-1;
    fr = nc.roots(find(nc.roots == ii));
    %tempsize(fr) = tempsize(fr) + 1;
    
    segt(ii) = sum(tempsize(1:ii));  
    segh(ii+1) = segt(ii)+1;
    
    root(fr) = 1;
  
end

tempsize(md.count.N) = md.interp.size(md.count.N);
segt(md.count.N) = sum(tempsize(1:md.count.N));

root(md.count.N) = 1;

%segh(end) = [];