function [val Gval] = neur_femfun(Gm,nc,md,p_const,...
                                  simd,VMEAS,filename)

tempGm = neur_cellularize(Gm,md);                              
p_conds.pas.Gm = neur_ctof(tempGm,md);                              
                                  
[M, K, L, b, md] = neur_assembleMKh(nc,md,p_const,p_conds);

eo = zeros(md.count.L,1);
eo(end) = 1;

Nt = simd.T/simd.dt+1;
Nlf = length(md.leaf);

io = neur_stimulus(md,simd);
 
% tl = length(simd.t);
% 
% % get initial potentials
% 
% simd.T2 = simd.T*10;
% simd.dt2 = simd.dt*1000;
% 
% LHS = M+simd.dt2*(L+K);
% [Lmat,Umat] = lu(LHS);
% 
% j = 2;
% %t(1) = 0;
% v(:,1) = ones(md.count.L,1)*parm.pas.v;
% 
% while (j-1)*simd.dt2 <= simd.T2
% 
%     %t(j) = (j-1)*simd.dt2;
% 
%     RHS = M*v(:,j-1)+simd.dt2*b;
% 
%     v(:,j) = Umat \ (Lmat \ RHS);
% 
%     j = j + 1;
% 
% end
% 
% solver

% solve the cable equation

LHS = M+simd.dt*(L+K);
[Lmat,Umat] = lu(LHS);

%v(:,1) = v(:,end);
v(:,1) = repmat(p_const.pas.v,md.count.L,1);

j = 2;

while (j-1)*simd.dt <= simd.T

    RHS = M*v(:,j-1)+simd.dt*b+io(j-1)*eo*simd.dt;

    v(:,j) = Umat \ (Lmat \ RHS);

    j = j + 1;

end

% evaluate the error objective

val = 0;

for jj = 1:Nlf

    ii = md.leaf(jj);
    
    err(jj,:) = v(md.segh(ii),:)-VMEAS(jj,:);
   
    % error function evaluate
    % trapezoidal rule
    
    val = val + ...
          (sum(err(jj,:).^2)-err(jj,1)^2/2-err(jj,Nt)^2/2)*...
          simd.dt/2;
    
end

err(Nlf+1,:) = v(md.segt(md.count.N),:)-VMEAS(Nlf+1,:);

% trapezoidal rule

val = val + ...
      (sum(err(Nlf+1,:).^2)-err(Nlf+1,1)^2/2-err(Nlf+1,Nt)^2/2)*...
      simd.dt/2;

% solve the ADJOINT CABLE EQUATION

j = simd.T/simd.dt;

V = repmat(0,md.count.L,Nt);

% solved backwards in time

while j >= 1

   RHS = M*V(:,j+1);
   
   for jj = 1:Nlf

       RHS(md.segh(md.leaf(jj))) = RHS(md.segh(md.leaf(jj)))+...
                                   simd.dt*(RHS(md.segh(md.leaf(jj))) - ...
                                   err(jj,j)./p_const.pas.Ra);
   
   end
   
   RHS(md.count.L) = RHS(md.count.L) + ...
                     simd.dt*(RHS(md.count.L) - err(end,j)./p_const.pas.Ra);
   
   V(:,j) = Umat \ (Lmat \ RHS);

   j = j - 1;

end

%feature('UseGenericOpenGL',1)

% compute conductance gradient

gvaldex = 1;

for kk = 1:md.count.N

    for jj = 1:md.grid.Cnumreg

        tmp = 0;
        s = md.segh(kk)+(jj-1)*md.grid.crat;
        f = s + md.grid.crat - 1;

        % compute on the regular part of the mesh
        
        for ii = s:f
            
            t1 = (v(ii,:)-p_const.pas.v).*V(ii,:)+...
                 (v(ii+1,:)-p_const.pas.v).*V(ii+1,:);
            t2 = (v(ii,:)-p_const.pas.v).*V(ii+1,:)+...
                 (v(ii+1,:)-p_const.pas.v).*V(ii,:);
            t3 = t1/3+t2/6;
            
            tmp = tmp + p_const.pas.Ra*(sum(t3)-t3(1)/2-t3(end)/2)*...
                        md.grid.hstep(kk)*simd.dt;
            
        end
        
        Gval(gvaldex) = tmp;
        gvaldex = gvaldex+1;
        
    end
 
    tmp = 0;
    s = f + 1;
    f = md.grid.Cnumleft(kk)+f;

    % compute on the leftover intervals

    for ii = s:f

        t1 = (v(ii,:)-p_const.pas.v).*V(ii,:)+...
             (v(ii+1,:)-p_const.pas.v).*V(ii+1,:);
        t2 = (v(ii,:)-p_const.pas.v).*V(ii+1,:)+...
             (v(ii+1,:)-p_const.pas.v).*V(ii,:);
        t3 = t1/3+t2/6;

        tmp = tmp + p_const.pas.Ra*(sum(t3)-t3(1)/2-t3(end)/2)*...
                    md.grid.hstep(kk)*simd.dt;

    end
    
    Gval(gvaldex) = tmp;
    gvaldex = gvaldex+1;

end

Gval = Gval/1e8;

% function [VMEAS tvec] = neur_get_recdata(md,filename)
% 
% out = neur_read(filename);
% out.v = out.v;
% 
% % get the potentials for all the terminal branches
% 
% Nt = size(out.v,2);
% 
% for jj = 1:length(md.leaf)
% 
%     ii = md.leaf(jj);
%     
%     VMEAS(jj,:) = out.v(md.segh(ii),:).*(1+randn(1,Nt)/2e3);
%     
% end
% 
% % get the potentials for soma end
% 
% VMEAS(end+1,:) = out.v(md.count.L,:).*(1+randn(1,Nt)/2e3);
% 
% tvec = out.t;

% coarse to fine function

function Gm = neur_ctof(CGm,md)

for jj = 1:md.count.N
    for ii = 1:md.grid.Cnumreg(jj)
        s = 1+(ii-1)*md.grid.crat;
        f = ii*md.grid.crat;
        Gm{jj}(s:f) = CGm{jj}(ii);
    end
    Gm{jj}(end+1:end+md.grid.Cnumleft(jj)) = CGm{jj}(end);
end

function Gm = neur_cellularize(VGm,md)

s = 1;
for jj = 1:md.count.N
 
    f = s+length(md.grid.Cxmid{jj})-1;
    Gm{jj} = VGm(s:f);
    s = f + 1;
    
end

function io = neur_stimulus(md,simd)

if simd.Is.end > simd.T
   simd.Is.end = simd.T;
end

% io = zeros(length(simd.t),1);
% 
% startindex = simd.Is.start/simd.dt;
% endindex = simd.Is.end/simd.dt;
% 
% io(startindex:endindex) = simd.Is.mag*1/(2*pi);

io = (3e-4)*max(simd.t-1,0).*exp(-max(simd.t-1,0)/2);
io = io/(2*pi);