close all
clear all

N= 50;
T = 10;

disp('exact solution...')

finedt = 0.025;

% if ~exist('vex')
%     vex = paxon_ex(50,finedt,10,4);
% end

load vex

finet = 0:finedt:10;
    
disp('done')

% read in data

filename = 'test.asc'

nc = neur_converterh(filename);

% construct morphology

md = neur_loadmorph_direct(nc,[N],1)

% parameters

p_const = neur_param_const(nc);
p_conds = neur_param_conds(nc,md);

% M K L

[M, K, L, b, md] = neur_assembleMKh(nc,md,p_const,p_conds);

save ncmd nc md

% Hines solver

disp('M,K,L ready')

disp('Solving system...')

simsteps = 0.1:-0.005:0.025;

for jj = 1:length(simsteps)

    % simulation time
    
    simd.dt = simsteps(jj);   % ms
    simd.T = T;    % ms

    simd.Is.start = 0;  % ms
    simd.Is.end = T;   % ms

    % current pulse

    simd.Is.mag = 1;  % uA
    %simd.Is.mag = 0;  % nA

    simd.t = 0:simd.dt:simd.T;

    % solve for v

    tic
    [v,t] = neur_solverh(M,K,L,b,md,p_const,p_conds,simd,filename);
    toc

    fprintf('\n')
    out = neur_read('test.asc')
    out.v = out.v(end:-1:1,:)-p_const.pas.v;
    
    for ii = 1:md.count.L
       
        pl = mkpl(t,out.v(ii,:));
        femv(ii,:) = ppval(pl,finet);
        
    end
   
    maxerr(jj) = max(max(abs(femv-vex)));

end

plot(simsteps,maxerr,'-o')
xlabel('time step (ms)'),ylabel('max|error| (mv)')
title(['patches: ' num2str(N+1)])